@@ -7,7 +7,7 @@
  */
 
 use crate::matrix_expression::MatrixExpression;
-use crate::size2::Size2;
+use crate::usize2::USize2;
 
 ///
 /// # Static-sized M x N matrix class.
@@ -25,8 +25,8 @@ pub struct Matrix<const M: usize, const N: usize, const TOTAL: usize> {
 }
 
 impl<const M: usize, const N: usize, const TOTAL: usize> MatrixExpression for Matrix<M, N, TOTAL> {
-    fn size(&self) -> Size2 {
-        return Size2::new(M, N);
+    fn size(&self) -> USize2 {
+        return USize2::new(M, N);
     }
 
     fn rows(&self) -> usize {
