@@ -6,7 +6,7 @@
  * // property of any third parties.
  */
 
-use crate::size2::Size2;
+use crate::usize2::USize2;
 use crate::operators::*;
 use crate::vector_expression::*;
 use std::ops::{Sub, Neg, Add, Mul, Div};
@@ -22,7 +22,7 @@ use std::ops::{Sub, Neg, Add, Mul, Div};
 ///
 pub trait MatrixExpression {
     /// Size of the matrix.
-    fn size(&self) -> Size2;
+    fn size(&self) -> USize2;
 
     /// Number of rows.
     fn rows(&self) -> usize;
@@ -61,8 +61,8 @@ impl MatrixConstant {
 
 impl MatrixExpression for MatrixConstant {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
-        return Size2::new(self.rows(), self.cols());
+    fn size(&self) -> USize2 {
+        return USize2::new(self.rows(), self.cols());
     }
 
     /// Number of rows.
@@ -104,8 +104,8 @@ impl MatrixIdentity {
 
 impl MatrixExpression for MatrixIdentity {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
-        return Size2::new(self._m, self._m);
+    fn size(&self) -> USize2 {
+        return USize2::new(self._m, self._m);
     }
 
     /// Number of rows.
@@ -157,7 +157,7 @@ impl<E: MatrixExpression, Op: UnaryOp> MatrixUnaryOp<E, Op> {
 
 impl<E: MatrixExpression, Op: UnaryOp> MatrixExpression for MatrixUnaryOp<E, Op> {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         return self._u.size();
     }
 
@@ -194,7 +194,7 @@ pub struct MatrixDiagonal<E: MatrixExpression> {
 
 impl<E: MatrixExpression> MatrixDiagonal<E> {
     /// Constructs diagonal matrix expression for given input expression.
-    /// \param is_diag - True for diagonal matrix, false for off-diagonal.
+    /// - parameter: is_diag - True for diagonal matrix, false for off-diagonal.
     pub fn new(u: E, is_diag: bool) -> MatrixDiagonal<E> {
         return MatrixDiagonal {
             _u: u,
@@ -205,7 +205,7 @@ impl<E: MatrixExpression> MatrixDiagonal<E> {
 
 impl<E: MatrixExpression> MatrixExpression for MatrixDiagonal<E> {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         return self._u.size();
     }
 
@@ -253,8 +253,8 @@ pub struct MatrixTriangular<E: MatrixExpression> {
 
 impl<E: MatrixExpression> MatrixTriangular<E> {
     /// Constructs triangular matrix expression for given input expression.
-    /// - param is_upper - True for upper tri matrix, false for lower tri matrix.
-    /// - param is_strict - True for strictly upper/lower triangular matrix.
+    /// - parameter: is_upper - True for upper tri matrix, false for lower tri matrix.
+    /// - parameter: is_strict - True for strictly upper/lower triangular matrix.
     pub fn new(u: E, is_upper: bool, is_strict: bool) -> MatrixTriangular<E> {
         return MatrixTriangular {
             _u: u,
@@ -266,7 +266,7 @@ impl<E: MatrixExpression> MatrixTriangular<E> {
 
 impl<E: MatrixExpression> MatrixExpression for MatrixTriangular<E> {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         return self._u.size();
     }
 
@@ -335,7 +335,7 @@ impl<E1: MatrixExpression, E2: MatrixExpression, Op: BinaryOp> MatrixBinaryOp<E1
 
 impl<E1: MatrixExpression, E2: MatrixExpression, Op: BinaryOp> MatrixExpression for MatrixBinaryOp<E1, E2, Op> {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         return self._v.size();
     }
 
@@ -384,7 +384,7 @@ impl<E: MatrixExpression, Op: BinaryOp> MatrixScalarBinaryOp<E, Op> {
 }
 
 impl<E: MatrixExpression, Op: BinaryOp> MatrixExpression for MatrixScalarBinaryOp<E, Op> {
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         return self._u.size();
     }
 
@@ -443,7 +443,7 @@ impl<ME: MatrixExpression, VE: VectorExpression> MatrixVectorMul<ME, VE> {
 }
 
 impl<ME: MatrixExpression, VE: VectorExpression> MatrixExpression for MatrixVectorMul<ME, VE> {
-    fn size(&self) -> Size2 {
+    fn size(&self) -> USize2 {
         unimplemented!();
     }
 
@@ -489,8 +489,8 @@ impl<E1: MatrixExpression, E2: MatrixExpression> MatrixMul<E1, E2> {
 
 impl<E1: MatrixExpression, E2: MatrixExpression> MatrixExpression for MatrixMul<E1, E2> {
     /// Size of the matrix.
-    fn size(&self) -> Size2 {
-        return Size2::new(self._u.rows(), self._v.cols());
+    fn size(&self) -> USize2 {
+        return USize2::new(self._u.rows(), self._v.cols());
     }
 
     /// Number of rows.
