@@ -14,8 +14,7 @@ use crate::bounding_box2::BoundingBox2D;
 use crate::ray2::Ray2D;
 use crate::nearest_neighbor_query_engine2::NearestNeighborQueryEngine2;
 use crate::intersection_query_engine2::IntersectionQueryEngine2;
-use std::cell::RefCell;
-use std::rc::Rc;
+use std::sync::{RwLock, Arc};
 
 ///
 /// # 2-D surface set.
@@ -27,14 +26,11 @@ use std::rc::Rc;
 pub struct SurfaceSet2 {
     _surfaces: Vec<Surface2Ptr>,
     _unbounded_surfaces: Vec<Surface2Ptr>,
-    _bvh: RefCell<Bvh2<Surface2Ptr>>,
-    _bvh_invalidated: RefCell<bool>,
+    _bvh: RwLock<Bvh2<Surface2Ptr>>,
+    _bvh_invalidated: RwLock<bool>,
 
-    /// Local-to-world transform.
-    pub transform: Transform2,
-
-    /// Flips normal.
-    pub is_normal_flipped: bool,
+    /// data from surface2
+    pub surface_data: Surface2Data,
 }
 
 impl SurfaceSet2 {
@@ -48,10 +44,9 @@ impl SurfaceSet2 {
         return SurfaceSet2 {
             _surfaces: vec![],
             _unbounded_surfaces: vec![],
-            _bvh: RefCell::new(Bvh2::new()),
-            _bvh_invalidated: RefCell::new(true),
-            transform: Transform2::new_default(),
-            is_normal_flipped: false,
+            _bvh: RwLock::new(Bvh2::new()),
+            _bvh_invalidated: RwLock::new(true),
+            surface_data: Surface2Data::new(None, None),
         };
     }
 
@@ -75,7 +70,7 @@ impl SurfaceSet2 {
                is_normal_flipped: Option<bool>) -> SurfaceSet2 {
         let mut unbounded_surfaces: Vec<Surface2Ptr> = vec![];
         for surface in &others {
-            if !surface.borrow().is_bounded() {
+            if !surface.read().unwrap().is_bounded() {
                 unbounded_surfaces.push(surface.clone());
             }
         }
@@ -83,10 +78,9 @@ impl SurfaceSet2 {
         return SurfaceSet2 {
             _surfaces: others.clone(),
             _unbounded_surfaces: unbounded_surfaces,
-            _bvh: RefCell::new(Bvh2::new()),
-            _bvh_invalidated: RefCell::new(true),
-            transform: transform.unwrap_or(Transform2::new_default()),
-            is_normal_flipped: is_normal_flipped.unwrap_or(false),
+            _bvh: RwLock::new(Bvh2::new()),
+            _bvh_invalidated: RwLock::new(true),
+            surface_data: Surface2Data::new(transform, is_normal_flipped),
         };
     }
 
@@ -125,7 +119,7 @@ impl SurfaceSet2 {
     /// ```
     pub fn add_surface(&mut self, surface: Surface2Ptr) {
         self._surfaces.push(surface.clone());
-        if !surface.borrow().is_bounded() {
+        if !surface.read().unwrap().is_bounded() {
             self._unbounded_surfaces.push(surface.clone());
         }
         self.invalidate_bvh();
@@ -133,22 +127,22 @@ impl SurfaceSet2 {
 
     /// must call it manually in Rust!
     pub fn build_bvh(&self) {
-        if *self._bvh_invalidated.borrow() {
+        if *self._bvh_invalidated.read().unwrap() {
             let mut surfs: Vec<Surface2Ptr> = Vec::new();
             let mut bounds: Vec<BoundingBox2D> = Vec::new();
             for i in 0..self._surfaces.len() {
-                if self._surfaces[i].borrow().is_bounded() {
+                if self._surfaces[i].read().unwrap().is_bounded() {
                     surfs.push(self._surfaces[i].clone());
-                    bounds.push(self._surfaces[i].borrow().bounding_box());
+                    bounds.push(self._surfaces[i].read().unwrap().bounding_box());
                 }
             }
-            self._bvh.borrow_mut().build(&surfs, &bounds);
-            self._bvh_invalidated.replace(false);
+            self._bvh.write().unwrap().build(&surfs, &bounds);
+            *(self._bvh_invalidated.write().unwrap()) = false;
         }
     }
 
     fn invalidate_bvh(&self) {
-        self._bvh_invalidated.replace(true);
+        *(self._bvh_invalidated.write().unwrap()) = true;
     }
 }
 
@@ -180,7 +174,7 @@ impl Surface2 for SurfaceSet2 {
     ///     let mut min_dist2 = f64::MAX;
     ///     let mut result = Vector2D::new_default();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_point(pt);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_point(pt);
     ///         let local_dist2 = pt.distance_squared_to(local_result);
     ///         if local_dist2 < min_dist2 {
     ///             min_dist2 = local_dist2;
@@ -218,25 +212,25 @@ impl Surface2 for SurfaceSet2 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface2Ptr, pt: &Vector2D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
         let mut result = Vector2D::new(f64::MAX, f64::MAX);
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {}
             Some(ptr) => {
-                result = ptr.borrow().closest_point(other_point);
+                result = ptr.read().unwrap().closest_point(other_point);
             }
         }
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
-                result = surface.borrow().closest_point(other_point);
+                result = surface.read().unwrap().closest_point(other_point);
             }
         }
 
@@ -263,7 +257,7 @@ impl Surface2 for SurfaceSet2 {
     ///                        .make_shared();
     ///     sset1.add_surface(sph.clone());
     ///
-    ///     answer.merge_box(&sph.borrow().bounding_box());
+    ///     answer.merge_box(&sph.read().unwrap().bounding_box());
     /// }
     /// assert_eq!(answer.lower_corner.is_similar(&sset1.bounding_box().lower_corner, Some(1e-9)), true);
     /// assert_eq!(answer.upper_corner.is_similar(&sset1.bounding_box().upper_corner, Some(1e-9)), true);
@@ -279,7 +273,7 @@ impl Surface2 for SurfaceSet2 {
     ///                        .make_shared();
     ///     sset2.add_surface(sph.clone());
     ///
-    ///     debug.merge_box(&sph.borrow().bounding_box());
+    ///     debug.merge_box(&sph.read().unwrap().bounding_box());
     /// }
     /// assert_eq!(answer.lower_corner.is_similar(&debug.lower_corner, Some(1e-9)), true);
     /// assert_eq!(answer.upper_corner.is_similar(&debug.upper_corner, Some(1e-9)), true);
@@ -290,7 +284,7 @@ impl Surface2 for SurfaceSet2 {
     fn bounding_box_local(&self) -> BoundingBox2D {
         self.build_bvh();
 
-        return self._bvh.borrow().bounding_box();
+        return self._bvh.read().unwrap().bounding_box();
     }
 
     /// ```
@@ -315,7 +309,7 @@ impl Surface2 for SurfaceSet2 {
     /// let brute_force_test = |ray:&Ray2D| {
     ///     let mut result = SurfaceRayIntersection2::new();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_intersection(ray);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_intersection(ray);
     ///         if local_result.distance < result.distance {
     ///             result = local_result;
     ///         }
@@ -358,11 +352,11 @@ impl Surface2 for SurfaceSet2 {
         self.build_bvh();
 
         let mut test_func = |surface: &Surface2Ptr, ray: &Ray2D| {
-            let result = surface.borrow().closest_intersection(ray);
+            let result = surface.read().unwrap().closest_intersection(ray);
             return result.distance;
         };
 
-        let query_result = self._bvh.borrow().closest_intersection(ray, &mut test_func);
+        let query_result = self._bvh.read().unwrap().closest_intersection(ray, &mut test_func);
         let mut result = SurfaceRayIntersection2::new();
         result.distance = query_result.distance;
         match query_result.item {
@@ -372,12 +366,12 @@ impl Surface2 for SurfaceSet2 {
             Some(ptr) => {
                 result.is_intersecting = true;
                 result.point = ray.point_at(query_result.distance);
-                result.normal = ptr.borrow().closest_normal(&result.point);
+                result.normal = ptr.read().unwrap().closest_normal(&result.point);
             }
         }
 
         for surface in &self._unbounded_surfaces {
-            let local_result = surface.borrow().closest_intersection(ray);
+            let local_result = surface.read().unwrap().closest_intersection(ray);
             if local_result.distance < result.distance {
                 result = local_result;
             }
@@ -409,8 +403,8 @@ impl Surface2 for SurfaceSet2 {
     ///     let mut min_dist2 = f64::MAX;
     ///     let mut result = Vector2D::new_default();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_normal(pt);
-    ///         let closest_pt = sset1.surface_at(i).borrow().closest_point(pt);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_normal(pt);
+    ///         let closest_pt = sset1.surface_at(i).read().unwrap().closest_point(pt);
     ///         let local_dist2 = pt.distance_squared_to(closest_pt);
     ///         if local_dist2 < min_dist2 {
     ///             min_dist2 = local_dist2;
@@ -448,25 +442,25 @@ impl Surface2 for SurfaceSet2 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface2Ptr, pt: &Vector2D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
         let mut result = Vector2D::new(1.0, 0.0);
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {}
             Some(ptr) => {
-                result = ptr.borrow().closest_normal(other_point);
+                result = ptr.read().unwrap().closest_normal(other_point);
             }
         }
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
-                result = surface.borrow().closest_normal(other_point);
+                result = surface.read().unwrap().closest_normal(other_point);
             }
         }
 
@@ -494,7 +488,7 @@ impl Surface2 for SurfaceSet2 {
     /// }
     /// let brute_force_test = |ray:&Ray2D| {
     ///     for i in 0..num_samples/2 {
-    ///         if sset1.surface_at(i).borrow().intersects(ray) {
+    ///         if sset1.surface_at(i).read().unwrap().intersects(ray) {
     ///             return true;
     ///         }
     ///     }
@@ -530,12 +524,12 @@ impl Surface2 for SurfaceSet2 {
         self.build_bvh();
 
         let mut test_func = |surface: &Surface2Ptr, ray: &Ray2D| {
-            return surface.borrow().intersects(ray);
+            return surface.read().unwrap().intersects(ray);
         };
 
-        let mut result = self._bvh.borrow().intersects_ray(ray_local, &mut test_func);
+        let mut result = self._bvh.read().unwrap().intersects_ray(ray_local, &mut test_func);
         for surface in &self._unbounded_surfaces {
-            result |= surface.borrow().intersects(ray_local);
+            result |= surface.read().unwrap().intersects(ray_local);
         }
 
         return result;
@@ -562,7 +556,7 @@ impl Surface2 for SurfaceSet2 {
     /// let brute_force_search = |pt:&Vector2D| {
     ///     let mut min_dist = f64::MAX;
     ///     for i in 0..num_samples/2 {
-    ///         let local_dist = sset1.surface_at(i).borrow().closest_distance(pt);
+    ///         let local_dist = sset1.surface_at(i).read().unwrap().closest_distance(pt);
     ///         if local_dist < min_dist {
     ///             min_dist = local_dist;
     ///         }
@@ -597,14 +591,14 @@ impl Surface2 for SurfaceSet2 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface2Ptr, pt: &Vector2D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
@@ -641,13 +635,13 @@ impl Surface2 for SurfaceSet2 {
     ///                           .with_transform(Transform2::new(offset, 0.0))
     ///                           .make_shared();
     ///
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector2D::new(0.5, 0.25) + offset)), true);
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector2D::new(0.5, 1.0) + offset)), true);
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector2D::new(0.5, 1.5) + offset)), false);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector2D::new(0.5, 0.25) + offset)), true);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector2D::new(0.5, 1.0) + offset)), true);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector2D::new(0.5, 1.5) + offset)), false);
     /// ```
     fn is_inside_local(&self, other_point_local: &Vector2D) -> bool {
         for surface in &self._surfaces {
-            if surface.borrow().is_inside(other_point_local) {
+            if surface.read().unwrap().is_inside(other_point_local) {
                 return true;
             }
         }
@@ -672,27 +666,27 @@ impl Surface2 for SurfaceSet2 {
     ///                           .with_transform(Transform2::new(Vector2D::new(1.0, 2.0), 0.0))
     ///                           .make_shared();
     ///
-    /// let bbox1 = surface_set.borrow().bounding_box();
+    /// let bbox1 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-0.5, 2.5), Vector2D::new(0.5, 3.5)).upper_corner, bbox1.upper_corner);
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-0.5, 2.5), Vector2D::new(0.5, 3.5)).lower_corner, bbox1.lower_corner);
     ///
-    /// surface_set.borrow_mut().transform = Transform2::new(Vector2D::new(3.0, -4.0), 0.0);
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox2 = surface_set.borrow().bounding_box();
+    /// surface_set.write().unwrap().transform = Transform2::new(Vector2D::new(3.0, -4.0), 0.0);
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox2 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(1.5, -3.5), Vector2D::new(2.5, -2.5)).upper_corner, bbox2.upper_corner);
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(1.5, -3.5), Vector2D::new(2.5, -2.5)).lower_corner, bbox2.lower_corner);
     ///
-    /// sphere.borrow_mut().transform = Transform2::new(Vector2D::new(-6.0, 9.0), 0.0);
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox3 = surface_set.borrow().bounding_box();
+    /// sphere.write().unwrap().transform = Transform2::new(Vector2D::new(-6.0, 9.0), 0.0);
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox3 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-4.5, 5.5), Vector2D::new(-3.5, 6.5)).upper_corner, bbox3.upper_corner);
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-4.5, 5.5), Vector2D::new(-3.5, 6.5)).lower_corner, bbox3.lower_corner);
     ///
     /// // Plane is unbounded. Total bbox should ignore it.
     /// let plane = Plane2::builder().with_normal(Vector2D::new(1.0, 0.0)).make_shared();
-    /// surface_set.borrow_mut().add_surface(plane);
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox4 = surface_set.borrow().bounding_box();
+    /// surface_set.write().unwrap().add_surface(plane);
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox4 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-4.5, 5.5), Vector2D::new(-3.5, 6.5)).upper_corner, bbox4.upper_corner);
     /// assert_eq!(BoundingBox2D::new(Vector2D::new(-4.5, 5.5), Vector2D::new(-3.5, 6.5)).lower_corner, bbox4.lower_corner);
     /// ```
@@ -724,9 +718,9 @@ impl Surface2 for SurfaceSet2 {
     ///                       .make_shared();
     ///
     /// let surface_set = SurfaceSet2::builder().with_surfaces(vec![plane, sphere]).make_shared();
-    /// assert_eq!(surface_set.borrow().is_bounded(), false);
+    /// assert_eq!(surface_set.read().unwrap().is_bounded(), false);
     ///
-    /// let cp = surface_set.borrow().closest_point(&Vector2D::new(0.5, 0.4));
+    /// let cp = surface_set.read().unwrap().closest_point(&Vector2D::new(0.5, 0.4));
     /// let answer = Vector2D::new(0.5, 0.5);
     ///
     /// assert_eq!(answer.is_similar(&cp, Some(1.0e-9)), true);
@@ -734,7 +728,7 @@ impl Surface2 for SurfaceSet2 {
     fn is_bounded(&self) -> bool {
         // All surfaces should be bounded.
         for surface in &self._surfaces {
-            if !surface.borrow().is_bounded() {
+            if !surface.read().unwrap().is_bounded() {
                 return false;
             }
         }
@@ -753,7 +747,7 @@ impl Surface2 for SurfaceSet2 {
     /// use vox_geometry_rust::unit_tests_utils::*;
     /// let surfaceSet = SurfaceSet2::builder().make_shared();
     ///
-    /// assert_eq!(surfaceSet.borrow().is_valid_geometry(), false);
+    /// assert_eq!(surfaceSet.read().unwrap().is_valid_geometry(), false);
     ///
     /// let domain = BoundingBox2D::new(Vector2D::new_default(), Vector2D::new(1.0, 2.0));
     ///
@@ -770,16 +764,16 @@ impl Surface2 for SurfaceSet2 {
     /// let surfaceSet2 =
     ///         SurfaceSet2::builder().with_surfaces(vec![plane, sphere]).make_shared();
     ///
-    /// assert_eq!(surfaceSet2.borrow().is_valid_geometry(), true);
+    /// assert_eq!(surfaceSet2.read().unwrap().is_valid_geometry(), true);
     ///
-    /// surfaceSet2.borrow_mut().add_surface(surfaceSet);
+    /// surfaceSet2.write().unwrap().add_surface(surfaceSet);
     ///
-    /// assert_eq!(surfaceSet2.borrow().is_valid_geometry(), false);
+    /// assert_eq!(surfaceSet2.read().unwrap().is_valid_geometry(), false);
     /// ```
     fn is_valid_geometry(&self) -> bool {
         // All surfaces should be valid.
         for surface in &self._surfaces {
-            if !surface.borrow().is_valid_geometry() {
+            if !surface.read().unwrap().is_valid_geometry() {
                 return false;
             }
         }
@@ -788,25 +782,13 @@ impl Surface2 for SurfaceSet2 {
         return !self._surfaces.is_empty();
     }
 
-    fn transform(&self) -> &Transform2 {
-        return &self.transform;
-    }
-
-    fn set_transform(&mut self, t: Transform2) {
-        self.transform = t;
-    }
-
-    fn is_normal_flipped(&self) -> bool {
-        return self.is_normal_flipped;
-    }
-
-    fn set_is_normal_flipped(&mut self, f: bool) {
-        self.is_normal_flipped = f;
+    fn view(&self) -> &Surface2Data {
+        return &self.surface_data;
     }
 }
 
 /// Shared pointer for the SurfaceSet2 type.
-pub type SurfaceSet2Ptr = Rc<RefCell<SurfaceSet2>>;
+pub type SurfaceSet2Ptr = Arc<RwLock<SurfaceSet2>>;
 
 ///
 /// # Front-end to create SurfaceSet2 objects step by step.
@@ -814,8 +796,7 @@ pub type SurfaceSet2Ptr = Rc<RefCell<SurfaceSet2>>;
 pub struct Builder {
     _surfaces: Vec<Surface2Ptr>,
 
-    _is_normal_flipped: bool,
-    _transform: Transform2,
+    _surface_data: Surface2Data,
 }
 
 impl Builder {
@@ -828,37 +809,26 @@ impl Builder {
     /// Builds SurfaceSet2.
     pub fn build(&mut self) -> SurfaceSet2 {
         return SurfaceSet2::new(self._surfaces.clone(),
-                                Some(self._transform.clone()),
-                                Some(self._is_normal_flipped));
+                                Some(self._surface_data.transform.clone()),
+                                Some(self._surface_data.is_normal_flipped));
     }
 
     /// Builds shared pointer of SurfaceSet2 instance.
     pub fn make_shared(&mut self) -> SurfaceSet2Ptr {
-        return SurfaceSet2Ptr::new(RefCell::new(self.build()));
+        return SurfaceSet2Ptr::new(RwLock::new(self.build()));
     }
 
     /// constructor
     pub fn new() -> Builder {
         return Builder {
             _surfaces: vec![],
-            _is_normal_flipped: false,
-            _transform: Transform2::new_default(),
+            _surface_data: Surface2Data::new(None, None),
         };
     }
 }
 
 impl SurfaceBuilderBase2 for Builder {
-    fn with_is_normal_flipped(&mut self, is_normal_flipped: bool) -> &mut Self {
-        self._is_normal_flipped = is_normal_flipped;
-        return self;
-    }
-
-    fn with_transform(&mut self, transform: Transform2) -> &mut Self {
-        self._transform = transform;
-        return self;
-    }
-
-    fn transform(&mut self) -> &mut Transform2 {
-        return &mut self._transform;
+    fn view(&mut self) -> &mut Surface2Data {
+        return &mut self._surface_data;
     }
 }
\ No newline at end of file
