@@ -14,8 +14,7 @@ use crate::bounding_box3::BoundingBox3D;
 use crate::ray3::Ray3D;
 use crate::nearest_neighbor_query_engine3::NearestNeighborQueryEngine3;
 use crate::intersection_query_engine3::IntersectionQueryEngine3;
-use std::cell::RefCell;
-use std::rc::Rc;
+use std::sync::{RwLock, Arc};
 
 ///
 /// # 3-D surface set.
@@ -27,14 +26,11 @@ use std::rc::Rc;
 pub struct SurfaceSet3 {
     _surfaces: Vec<Surface3Ptr>,
     _unbounded_surfaces: Vec<Surface3Ptr>,
-    _bvh: RefCell<Bvh3<Surface3Ptr>>,
-    _bvh_invalidated: RefCell<bool>,
+    _bvh: RwLock<Bvh3<Surface3Ptr>>,
+    _bvh_invalidated: RwLock<bool>,
 
-    /// Local-to-world transform.
-    pub transform: Transform3,
-
-    /// Flips normal.
-    pub is_normal_flipped: bool,
+    /// data from surface3
+    pub surface_data: Surface3Data,
 }
 
 impl SurfaceSet3 {
@@ -48,10 +44,9 @@ impl SurfaceSet3 {
         return SurfaceSet3 {
             _surfaces: vec![],
             _unbounded_surfaces: vec![],
-            _bvh: RefCell::new(Bvh3::new()),
-            _bvh_invalidated: RefCell::new(true),
-            transform: Transform3::new_default(),
-            is_normal_flipped: false,
+            _bvh: RwLock::new(Bvh3::new()),
+            _bvh_invalidated: RwLock::new(true),
+            surface_data: Surface3Data::new(None, None),
         };
     }
 
@@ -76,7 +71,7 @@ impl SurfaceSet3 {
                is_normal_flipped: Option<bool>) -> SurfaceSet3 {
         let mut unbounded_surfaces: Vec<Surface3Ptr> = vec![];
         for surface in &others {
-            if !surface.borrow().is_bounded() {
+            if !surface.read().unwrap().is_bounded() {
                 unbounded_surfaces.push(surface.clone());
             }
         }
@@ -84,10 +79,9 @@ impl SurfaceSet3 {
         return SurfaceSet3 {
             _surfaces: others.clone(),
             _unbounded_surfaces: unbounded_surfaces,
-            _bvh: RefCell::new(Bvh3::new()),
-            _bvh_invalidated: RefCell::new(true),
-            transform: transform.unwrap_or(Transform3::new_default()),
-            is_normal_flipped: is_normal_flipped.unwrap_or(false),
+            _bvh: RwLock::new(Bvh3::new()),
+            _bvh_invalidated: RwLock::new(true),
+            surface_data: Surface3Data::new(transform, is_normal_flipped),
         };
     }
 
@@ -126,7 +120,7 @@ impl SurfaceSet3 {
     /// ```
     pub fn add_surface(&mut self, surface: Surface3Ptr) {
         self._surfaces.push(surface.clone());
-        if !surface.borrow().is_bounded() {
+        if !surface.read().unwrap().is_bounded() {
             self._unbounded_surfaces.push(surface.clone());
         }
         self.invalidate_bvh();
@@ -134,22 +128,22 @@ impl SurfaceSet3 {
 
     /// must call it manually in Rust!
     pub fn build_bvh(&self) {
-        if *self._bvh_invalidated.borrow() {
+        if *self._bvh_invalidated.read().unwrap() {
             let mut surfs: Vec<Surface3Ptr> = Vec::new();
             let mut bounds: Vec<BoundingBox3D> = Vec::new();
             for i in 0..self._surfaces.len() {
-                if self._surfaces[i].borrow().is_bounded() {
+                if self._surfaces[i].read().unwrap().is_bounded() {
                     surfs.push(self._surfaces[i].clone());
-                    bounds.push(self._surfaces[i].borrow().bounding_box());
+                    bounds.push(self._surfaces[i].read().unwrap().bounding_box());
                 }
             }
-            self._bvh.borrow_mut().build(&surfs, &bounds);
-            self._bvh_invalidated.replace(false);
+            self._bvh.write().unwrap().build(&surfs, &bounds);
+            *(self._bvh_invalidated.write().unwrap()) = false;
         }
     }
 
     fn invalidate_bvh(&self) {
-        self._bvh_invalidated.replace(true);
+        *(self._bvh_invalidated.write().unwrap()) = true;
     }
 }
 
@@ -182,7 +176,7 @@ impl Surface3 for SurfaceSet3 {
     ///     let mut min_dist3 = f64::MAX;
     ///     let mut result = Vector3D::new_default();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_point(pt);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_point(pt);
     ///         let local_dist3 = pt.distance_squared_to(local_result);
     ///         if local_dist3 < min_dist3 {
     ///             min_dist3 = local_dist3;
@@ -220,25 +214,25 @@ impl Surface3 for SurfaceSet3 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface3Ptr, pt: &Vector3D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
         let mut result = Vector3D::new(f64::MAX, f64::MAX, f64::MAX);
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {}
             Some(ptr) => {
-                result = ptr.borrow().closest_point(other_point);
+                result = ptr.read().unwrap().closest_point(other_point);
             }
         }
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
-                result = surface.borrow().closest_point(other_point);
+                result = surface.read().unwrap().closest_point(other_point);
             }
         }
 
@@ -265,7 +259,7 @@ impl Surface3 for SurfaceSet3 {
     ///                        .make_shared();
     ///     sset1.add_surface(sph.clone());
     ///
-    ///     answer.merge_box(&sph.borrow().bounding_box());
+    ///     answer.merge_box(&sph.read().unwrap().bounding_box());
     /// }
     /// assert_eq!(answer.lower_corner.is_similar(&sset1.bounding_box().lower_corner, Some(1e-9)), true);
     /// assert_eq!(answer.upper_corner.is_similar(&sset1.bounding_box().upper_corner, Some(1e-9)), true);
@@ -281,7 +275,7 @@ impl Surface3 for SurfaceSet3 {
     ///                        .make_shared();
     ///     sset3.add_surface(sph.clone());
     ///
-    ///     debug.merge_box(&sph.borrow().bounding_box());
+    ///     debug.merge_box(&sph.read().unwrap().bounding_box());
     /// }
     /// assert_eq!(answer.lower_corner.is_similar(&debug.lower_corner, Some(1e-9)), true);
     /// assert_eq!(answer.upper_corner.is_similar(&debug.upper_corner, Some(1e-9)), true);
@@ -292,7 +286,7 @@ impl Surface3 for SurfaceSet3 {
     fn bounding_box_local(&self) -> BoundingBox3D {
         self.build_bvh();
 
-        return self._bvh.borrow().bounding_box();
+        return self._bvh.read().unwrap().bounding_box();
     }
 
     /// ```
@@ -317,7 +311,7 @@ impl Surface3 for SurfaceSet3 {
     /// let brute_force_test = |ray:&Ray3D| {
     ///     let mut result = SurfaceRayIntersection3::new();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_intersection(ray);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_intersection(ray);
     ///         if local_result.distance < result.distance {
     ///             result = local_result;
     ///         }
@@ -360,11 +354,11 @@ impl Surface3 for SurfaceSet3 {
         self.build_bvh();
 
         let mut test_func = |surface: &Surface3Ptr, ray: &Ray3D| {
-            let result = surface.borrow().closest_intersection(ray);
+            let result = surface.read().unwrap().closest_intersection(ray);
             return result.distance;
         };
 
-        let query_result = self._bvh.borrow().closest_intersection(ray, &mut test_func);
+        let query_result = self._bvh.read().unwrap().closest_intersection(ray, &mut test_func);
         let mut result = SurfaceRayIntersection3::new();
         result.distance = query_result.distance;
         match query_result.item {
@@ -374,12 +368,12 @@ impl Surface3 for SurfaceSet3 {
             Some(ptr) => {
                 result.is_intersecting = true;
                 result.point = ray.point_at(query_result.distance);
-                result.normal = ptr.borrow().closest_normal(&result.point);
+                result.normal = ptr.read().unwrap().closest_normal(&result.point);
             }
         }
 
         for surface in &self._unbounded_surfaces {
-            let local_result = surface.borrow().closest_intersection(ray);
+            let local_result = surface.read().unwrap().closest_intersection(ray);
             if local_result.distance < result.distance {
                 result = local_result;
             }
@@ -411,8 +405,8 @@ impl Surface3 for SurfaceSet3 {
     ///     let mut min_dist3 = f64::MAX;
     ///     let mut result = Vector3D::new_default();
     ///     for i in 0..num_samples/2 {
-    ///         let local_result = sset1.surface_at(i).borrow().closest_normal(pt);
-    ///         let closest_pt = sset1.surface_at(i).borrow().closest_point(pt);
+    ///         let local_result = sset1.surface_at(i).read().unwrap().closest_normal(pt);
+    ///         let closest_pt = sset1.surface_at(i).read().unwrap().closest_point(pt);
     ///         let local_dist3 = pt.distance_squared_to(closest_pt);
     ///         if local_dist3 < min_dist3 {
     ///             min_dist3 = local_dist3;
@@ -450,25 +444,25 @@ impl Surface3 for SurfaceSet3 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface3Ptr, pt: &Vector3D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
         let mut result = Vector3D::new(1.0, 0.0, 0.0);
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {}
             Some(ptr) => {
-                result = ptr.borrow().closest_normal(other_point);
+                result = ptr.read().unwrap().closest_normal(other_point);
             }
         }
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
-                result = surface.borrow().closest_normal(other_point);
+                result = surface.read().unwrap().closest_normal(other_point);
             }
         }
 
@@ -496,7 +490,7 @@ impl Surface3 for SurfaceSet3 {
     /// }
     /// let brute_force_test = |ray:&Ray3D| {
     ///     for i in 0..num_samples/2 {
-    ///         if sset1.surface_at(i).borrow().intersects(ray) {
+    ///         if sset1.surface_at(i).read().unwrap().intersects(ray) {
     ///             return true;
     ///         }
     ///     }
@@ -532,12 +526,12 @@ impl Surface3 for SurfaceSet3 {
         self.build_bvh();
 
         let mut test_func = |surface: &Surface3Ptr, ray: &Ray3D| {
-            return surface.borrow().intersects(ray);
+            return surface.read().unwrap().intersects(ray);
         };
 
-        let mut result = self._bvh.borrow().intersects_ray(ray_local, &mut test_func);
+        let mut result = self._bvh.read().unwrap().intersects_ray(ray_local, &mut test_func);
         for surface in &self._unbounded_surfaces {
-            result |= surface.borrow().intersects(ray_local);
+            result |= surface.read().unwrap().intersects(ray_local);
         }
 
         return result;
@@ -564,7 +558,7 @@ impl Surface3 for SurfaceSet3 {
     /// let brute_force_search = |pt:&Vector3D| {
     ///     let mut min_dist = f64::MAX;
     ///     for i in 0..num_samples/2 {
-    ///         let local_dist = sset1.surface_at(i).borrow().closest_distance(pt);
+    ///         let local_dist = sset1.surface_at(i).read().unwrap().closest_distance(pt);
     ///         if local_dist < min_dist {
     ///             min_dist = local_dist;
     ///         }
@@ -599,14 +593,14 @@ impl Surface3 for SurfaceSet3 {
         self.build_bvh();
 
         let mut distance_func = |surface: &Surface3Ptr, pt: &Vector3D| {
-            return surface.borrow().closest_distance(pt);
+            return surface.read().unwrap().closest_distance(pt);
         };
 
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
 
         let mut min_dist = query_result.distance;
         for surface in &self._unbounded_surfaces {
-            let pt = surface.borrow().closest_point(other_point);
+            let pt = surface.read().unwrap().closest_point(other_point);
             let dist = pt.distance_to(*other_point);
             if dist < min_dist {
                 min_dist = dist;
@@ -644,13 +638,13 @@ impl Surface3 for SurfaceSet3 {
     ///                           .with_transform(Transform3::new(offset, QuaternionD::new_default()))
     ///                           .make_shared();
     ///
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector3D::new(0.5, 0.25, 0.5) + offset)), true);
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector3D::new(0.5, 1.0, 0.5) + offset)), true);
-    /// assert_eq!(surface_set.borrow().is_inside(&(Vector3D::new(0.5, 1.5, 0.5) + offset)), false);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector3D::new(0.5, 0.25, 0.5) + offset)), true);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector3D::new(0.5, 1.0, 0.5) + offset)), true);
+    /// assert_eq!(surface_set.read().unwrap().is_inside(&(Vector3D::new(0.5, 1.5, 0.5) + offset)), false);
     /// ```
     fn is_inside_local(&self, other_point_local: &Vector3D) -> bool {
         for surface in &self._surfaces {
-            if surface.borrow().is_inside(other_point_local) {
+            if surface.read().unwrap().is_inside(other_point_local) {
                 return true;
             }
         }
@@ -676,27 +670,27 @@ impl Surface3 for SurfaceSet3 {
     ///                           .with_transform(Transform3::new(Vector3D::new(1.0, 2.0, -1.0), QuaternionD::new_default()))
     ///                           .make_shared();
     ///
-    /// let bbox1 = surface_set.borrow().bounding_box();
+    /// let bbox1 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-0.5, 2.5, 0.5), Vector3D::new(0.5, 3.5, 1.5)).upper_corner, bbox1.upper_corner);
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-0.5, 2.5, 0.5), Vector3D::new(0.5, 3.5, 1.5)).lower_corner, bbox1.lower_corner);
     ///
-    /// surface_set.borrow_mut().transform = Transform3::new(Vector3D::new(3.0, -4.0, 7.0), QuaternionD::new_default());
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox3 = surface_set.borrow().bounding_box();
+    /// surface_set.write().unwrap().transform = Transform3::new(Vector3D::new(3.0, -4.0, 7.0), QuaternionD::new_default());
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox3 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(1.5, -3.5, 8.5), Vector3D::new(2.5, -2.5, 9.5)).upper_corner, bbox3.upper_corner);
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(1.5, -3.5, 8.5), Vector3D::new(2.5, -2.5, 9.5)).lower_corner, bbox3.lower_corner);
     ///
-    /// sphere.borrow_mut().transform = Transform3::new(Vector3D::new(-6.0, 9.0, 2.0), QuaternionD::new_default());
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox3 = surface_set.borrow().bounding_box();
+    /// sphere.write().unwrap().transform = Transform3::new(Vector3D::new(-6.0, 9.0, 2.0), QuaternionD::new_default());
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox3 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-4.5, 5.5, 10.5), Vector3D::new(-3.5, 6.5, 11.5)).upper_corner, bbox3.upper_corner);
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-4.5, 5.5, 10.5), Vector3D::new(-3.5, 6.5, 11.5)).lower_corner, bbox3.lower_corner);
     ///
     /// // Plane is unbounded. Total bbox should ignore it.
     /// let plane = Plane3::builder().with_normal(Vector3D::new(1.0, 0.0, 0.0)).make_shared();
-    /// surface_set.borrow_mut().add_surface(plane);
-    /// surface_set.borrow_mut().update_query_engine();
-    /// let bbox4 = surface_set.borrow().bounding_box();
+    /// surface_set.write().unwrap().add_surface(plane);
+    /// surface_set.write().unwrap().update_query_engine();
+    /// let bbox4 = surface_set.read().unwrap().bounding_box();
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-4.5, 5.5, 10.5), Vector3D::new(-3.5, 6.5, 11.5)).upper_corner, bbox4.upper_corner);
     /// assert_eq!(BoundingBox3D::new(Vector3D::new(-4.5, 5.5, 10.5), Vector3D::new(-3.5, 6.5, 11.5)).lower_corner, bbox4.lower_corner);
     /// ```
@@ -728,9 +722,9 @@ impl Surface3 for SurfaceSet3 {
     ///                       .make_shared();
     ///
     /// let surface_set = SurfaceSet3::builder().with_surfaces(vec![plane, sphere]).make_shared();
-    /// assert_eq!(surface_set.borrow().is_bounded(), false);
+    /// assert_eq!(surface_set.read().unwrap().is_bounded(), false);
     ///
-    /// let cp = surface_set.borrow().closest_point(&Vector3D::new(0.5, 0.4, 0.5));
+    /// let cp = surface_set.read().unwrap().closest_point(&Vector3D::new(0.5, 0.4, 0.5));
     /// let answer = Vector3D::new(0.5, 0.5, 0.5);
     ///
     /// assert_eq!(answer.is_similar(&cp, Some(1.0e-9)), true);
@@ -738,7 +732,7 @@ impl Surface3 for SurfaceSet3 {
     fn is_bounded(&self) -> bool {
         // All surfaces should be bounded.
         for surface in &self._surfaces {
-            if !surface.borrow().is_bounded() {
+            if !surface.read().unwrap().is_bounded() {
                 return false;
             }
         }
@@ -757,7 +751,7 @@ impl Surface3 for SurfaceSet3 {
     /// use vox_geometry_rust::unit_tests_utils::*;
     /// let surfaceSet = SurfaceSet3::builder().make_shared();
     ///
-    /// assert_eq!(surfaceSet.borrow().is_valid_geometry(), false);
+    /// assert_eq!(surfaceSet.read().unwrap().is_valid_geometry(), false);
     ///
     /// let domain = BoundingBox3D::new(Vector3D::new_default(), Vector3D::new(1.0, 2.0, 1.0));
     ///
@@ -774,16 +768,16 @@ impl Surface3 for SurfaceSet3 {
     /// let surfaceSet3 =
     ///         SurfaceSet3::builder().with_surfaces(vec![plane, sphere]).make_shared();
     ///
-    /// assert_eq!(surfaceSet3.borrow().is_valid_geometry(), true);
+    /// assert_eq!(surfaceSet3.read().unwrap().is_valid_geometry(), true);
     ///
-    /// surfaceSet3.borrow_mut().add_surface(surfaceSet);
+    /// surfaceSet3.write().unwrap().add_surface(surfaceSet);
     ///
-    /// assert_eq!(surfaceSet3.borrow().is_valid_geometry(), false);
+    /// assert_eq!(surfaceSet3.read().unwrap().is_valid_geometry(), false);
     /// ```
     fn is_valid_geometry(&self) -> bool {
         // All surfaces should be valid.
         for surface in &self._surfaces {
-            if !surface.borrow().is_valid_geometry() {
+            if !surface.read().unwrap().is_valid_geometry() {
                 return false;
             }
         }
@@ -792,25 +786,13 @@ impl Surface3 for SurfaceSet3 {
         return !self._surfaces.is_empty();
     }
 
-    fn transform(&self) -> &Transform3 {
-        return &self.transform;
-    }
-
-    fn set_transform(&mut self, t: Transform3) {
-        self.transform = t;
-    }
-
-    fn is_normal_flipped(&self) -> bool {
-        return self.is_normal_flipped;
-    }
-
-    fn set_is_normal_flipped(&mut self, f: bool) {
-        self.is_normal_flipped = f;
+    fn view(&self) -> &Surface3Data {
+        return &self.surface_data;
     }
 }
 
 /// Shared pointer for the SurfaceSet3 type.
-pub type SurfaceSet3Ptr = Rc<RefCell<SurfaceSet3>>;
+pub type SurfaceSet3Ptr = Arc<RwLock<SurfaceSet3>>;
 
 ///
 /// # Front-end to create SurfaceSet3 objects step by step.
@@ -818,8 +800,7 @@ pub type SurfaceSet3Ptr = Rc<RefCell<SurfaceSet3>>;
 pub struct Builder {
     _surfaces: Vec<Surface3Ptr>,
 
-    _is_normal_flipped: bool,
-    _transform: Transform3,
+    _surface_data: Surface3Data,
 }
 
 impl Builder {
@@ -832,37 +813,26 @@ impl Builder {
     /// Builds SurfaceSet3.
     pub fn build(&mut self) -> SurfaceSet3 {
         return SurfaceSet3::new(self._surfaces.clone(),
-                                Some(self._transform.clone()),
-                                Some(self._is_normal_flipped));
+                                Some(self._surface_data.transform.clone()),
+                                Some(self._surface_data.is_normal_flipped));
     }
 
     /// Builds shared pointer of SurfaceSet3 instance.
     pub fn make_shared(&mut self) -> SurfaceSet3Ptr {
-        return SurfaceSet3Ptr::new(RefCell::new(self.build()));
+        return SurfaceSet3Ptr::new(RwLock::new(self.build()));
     }
 
     /// constructor
     pub fn new() -> Builder {
         return Builder {
             _surfaces: vec![],
-            _is_normal_flipped: false,
-            _transform: Transform3::new_default(),
+            _surface_data: Surface3Data::new(None, None),
         };
     }
 }
 
 impl SurfaceBuilderBase3 for Builder {
-    fn with_is_normal_flipped(&mut self, is_normal_flipped: bool) -> &mut Self {
-        self._is_normal_flipped = is_normal_flipped;
-        return self;
-    }
-
-    fn with_transform(&mut self, transform: Transform3) -> &mut Self {
-        self._transform = transform;
-        return self;
-    }
-
-    fn transform(&mut self) -> &mut Transform3 {
-        return &mut self._transform;
+    fn view(&mut self) -> &mut Surface3Data {
+        return &mut self._surface_data;
     }
 }
\ No newline at end of file
