@@ -8,9 +8,9 @@
 
 use crate::vector3::Vector3D;
 use crate::vector2::Vector2D;
-use crate::size3::Size3;
+use crate::usize3::USize3;
 use crate::transform3::Transform3;
-use crate::surface3::{SurfaceBuilderBase3, Surface3, SurfaceRayIntersection3};
+use crate::surface3::*;
 use crate::bvh3::Bvh3;
 use crate::triangle3::Triangle3;
 use crate::quaternion::QuaternionD;
@@ -22,8 +22,7 @@ use crate::matrix3x3::Matrix3x3D;
 use rayon::iter::ParallelIterator;
 use rayon::prelude::IntoParallelRefMutIterator;
 use std::mem::swap;
-use std::cell::RefCell;
-use std::rc::Rc;
+use std::sync::{RwLock, Arc};
 use std::ops::Add;
 use std::io::{Cursor, BufWriter, Write, Result};
 use std::borrow::BorrowMut;
@@ -96,22 +95,19 @@ pub struct TriangleMesh3 {
     _points: Vec<Vector3D>,
     _normals: Vec<Vector3D>,
     _uvs: Vec<Vector2D>,
-    _point_indices: Vec<Size3>,
-    _normal_indices: Vec<Size3>,
-    _uv_indices: Vec<Size3>,
+    _point_indices: Vec<USize3>,
+    _normal_indices: Vec<USize3>,
+    _uv_indices: Vec<USize3>,
 
-    _bvh: RefCell<Bvh3<usize>>,
-    _bvh_invalidated: RefCell<bool>,
+    _bvh: RwLock<Bvh3<usize>>,
+    _bvh_invalidated: RwLock<bool>,
 
-    _wn_area_weighted_normal_sums: RefCell<Vec<Vector3D>>,
-    _wn_area_weighted_avg_positions: RefCell<Vec<Vector3D>>,
-    _wn_invalidated: RefCell<bool>,
+    _wn_area_weighted_normal_sums: RwLock<Vec<Vector3D>>,
+    _wn_area_weighted_avg_positions: RwLock<Vec<Vector3D>>,
+    _wn_invalidated: RwLock<bool>,
 
-    /// Local-to-world transform.
-    pub transform: Transform3,
-
-    /// Flips normal.
-    pub is_normal_flipped: bool,
+    /// data from surface3
+    pub surface_data: Surface3Data,
 }
 
 impl TriangleMesh3 {
@@ -133,20 +129,19 @@ impl TriangleMesh3 {
             _point_indices: vec![],
             _normal_indices: vec![],
             _uv_indices: vec![],
-            _bvh: RefCell::new(Bvh3::new()),
-            _bvh_invalidated: RefCell::new(true),
-            _wn_area_weighted_normal_sums: RefCell::new(vec![]),
-            _wn_area_weighted_avg_positions: RefCell::new(vec![]),
-            _wn_invalidated: RefCell::new(true),
-            transform: transform.unwrap_or(Transform3::new_default()),
-            is_normal_flipped: is_normal_flipped.unwrap_or(false),
+            _bvh: RwLock::new(Bvh3::new()),
+            _bvh_invalidated: RwLock::new(true),
+            _wn_area_weighted_normal_sums: RwLock::new(vec![]),
+            _wn_area_weighted_avg_positions: RwLock::new(vec![]),
+            _wn_invalidated: RwLock::new(true),
+            surface_data: Surface3Data::new(transform, is_normal_flipped),
         };
     }
 
     /// Constructs mesh with points, normals, uvs, and their indices.
     pub fn new(points: Vec<Vector3D>, normals: Vec<Vector3D>,
-               uvs: Vec<Vector2D>, point_indices: Vec<Size3>,
-               normal_indices: Vec<Size3>, uv_indices: Vec<Size3>,
+               uvs: Vec<Vector2D>, point_indices: Vec<USize3>,
+               normal_indices: Vec<USize3>, uv_indices: Vec<USize3>,
                transform: Option<Transform3>,
                is_normal_flipped: Option<bool>) -> TriangleMesh3 {
         return TriangleMesh3 {
@@ -156,13 +151,12 @@ impl TriangleMesh3 {
             _point_indices: point_indices,
             _normal_indices: normal_indices,
             _uv_indices: uv_indices,
-            _bvh: RefCell::new(Bvh3::new()),
-            _bvh_invalidated: RefCell::new(true),
-            _wn_area_weighted_normal_sums: RefCell::new(vec![]),
-            _wn_area_weighted_avg_positions: RefCell::new(vec![]),
-            _wn_invalidated: RefCell::new(true),
-            transform: transform.unwrap_or(Transform3::new_default()),
-            is_normal_flipped: is_normal_flipped.unwrap_or(false),
+            _bvh: RwLock::new(Bvh3::new()),
+            _bvh_invalidated: RwLock::new(true),
+            _wn_area_weighted_normal_sums: RwLock::new(vec![]),
+            _wn_area_weighted_avg_positions: RwLock::new(vec![]),
+            _wn_invalidated: RwLock::new(true),
+            surface_data: Surface3Data::new(transform, is_normal_flipped),
         };
     }
 
@@ -257,32 +251,32 @@ impl TriangleMesh3 {
     }
 
     /// Returns constant reference to the point indices of i-th triangle.
-    pub fn point_index(&self, i: usize) -> Size3 {
+    pub fn point_index(&self, i: usize) -> USize3 {
         return self._point_indices[i];
     }
 
     /// Returns reference to the point indices of i-th triangle.
-    pub fn point_index_mut(&mut self, i: usize) -> Size3 {
+    pub fn point_index_mut(&mut self, i: usize) -> USize3 {
         return self._point_indices[i];
     }
 
     /// Returns constant reference to the normal indices of i-th triangle.
-    pub fn normal_index(&self, i: usize) -> Size3 {
+    pub fn normal_index(&self, i: usize) -> USize3 {
         return self._normal_indices[i];
     }
 
     /// Returns reference to the normal indices of i-th triangle.
-    pub fn normal_index_mut(&mut self, i: usize) -> Size3 {
+    pub fn normal_index_mut(&mut self, i: usize) -> USize3 {
         return self._normal_indices[i];
     }
 
     /// Returns constant reference to the UV indices of i-th triangle.
-    pub fn uv_index(&self, i: usize) -> Size3 {
+    pub fn uv_index(&self, i: usize) -> USize3 {
         return self._uv_indices[i];
     }
 
     /// Returns reference to the UV indices of i-th triangle.
-    pub fn uv_index_mut(&mut self, i: usize) -> Size3 {
+    pub fn uv_index_mut(&mut self, i: usize) -> USize3 {
         return self._uv_indices[i];
     }
 
@@ -355,28 +349,28 @@ impl TriangleMesh3 {
     }
 
     /// Adds a triangle with points.
-    pub fn add_point_triangle(&mut self, new_point_indices: Size3) {
+    pub fn add_point_triangle(&mut self, new_point_indices: USize3) {
         self._point_indices.push(new_point_indices);
         self.invalidate_cache();
     }
 
     /// Adds a triangle with normal.
-    pub fn add_normal_triangle(&mut self, new_normal_indices: Size3) {
+    pub fn add_normal_triangle(&mut self, new_normal_indices: USize3) {
         self._normal_indices.push(new_normal_indices);
 
         self.invalidate_cache();
     }
 
     /// Adds a triangle with UV.
-    pub fn add_uv_triangle(&mut self, new_uv_indices: Size3) {
+    pub fn add_uv_triangle(&mut self, new_uv_indices: USize3) {
         self._uv_indices.push(new_uv_indices);
 
         self.invalidate_cache();
     }
 
     /// Adds a triangle with point and normal.
-    pub fn add_point_normal_triangle(&mut self, new_point_indices: Size3,
-                                     new_normal_indices: Size3) {
+    pub fn add_point_normal_triangle(&mut self, new_point_indices: USize3,
+                                     new_normal_indices: USize3) {
         self._point_indices.push(new_point_indices);
         self._normal_indices.push(new_normal_indices);
 
@@ -384,9 +378,9 @@ impl TriangleMesh3 {
     }
 
     /// Adds a triangle with point, normal, and UV.
-    pub fn add_point_uv_normal_triangle(&mut self, new_point_indices: Size3,
-                                        new_uv_indices: Size3,
-                                        new_normal_indices: Size3) {
+    pub fn add_point_uv_normal_triangle(&mut self, new_point_indices: USize3,
+                                        new_uv_indices: USize3,
+                                        new_normal_indices: USize3) {
         self._point_indices.push(new_point_indices);
         self._normal_indices.push(new_normal_indices);
         self._uv_indices.push(new_uv_indices);
@@ -395,8 +389,8 @@ impl TriangleMesh3 {
     }
 
     /// Adds a triangle with point and UV.
-    pub fn add_point_uv_triangle(&mut self, new_point_indices: Size3,
-                                 new_uv_indices: Size3) {
+    pub fn add_point_uv_triangle(&mut self, new_point_indices: USize3,
+                                 new_uv_indices: USize3) {
         self._point_indices.push(new_point_indices);
         self._uv_indices.push(new_uv_indices);
 
@@ -408,9 +402,9 @@ impl TriangleMesh3 {
         let v_start = self._points.len();
         let n_start = self._normals.len();
         let t_start = self._uvs.len();
-        let mut new_point_indices = Size3::new_default();
-        let mut new_normal_indices = Size3::new_default();
-        let mut new_uv_indices = Size3::new_default();
+        let mut new_point_indices = USize3::new_default();
+        let mut new_normal_indices = USize3::new_default();
+        let mut new_uv_indices = USize3::new_default();
         for i in 0..3 {
             self._points.push(tri.points[i]);
             self._normals.push(tri.normals[i]);
@@ -552,12 +546,12 @@ impl TriangleMesh3 {
 
 impl TriangleMesh3 {
     fn invalidate_cache(&self) {
-        self._bvh_invalidated.replace(true);
-        self._wn_invalidated.replace(true);
+        *(self._bvh_invalidated.write().unwrap()) = true;
+        *(self._wn_invalidated.write().unwrap()) = true;
     }
 
     fn build_bvh(&self) {
-        if *self._bvh_invalidated.borrow() {
+        if *self._bvh_invalidated.read().unwrap() {
             let n_tris = self.number_of_triangles();
             let mut ids: Vec<usize> = Vec::new();
             ids.resize(n_tris, 0);
@@ -567,29 +561,29 @@ impl TriangleMesh3 {
                 ids[i] = i;
                 bounds[i] = self.triangle(i).bounding_box();
             }
-            self._bvh.borrow_mut().build(&ids, &bounds);
-            self._bvh_invalidated.replace(false);
+            self._bvh.write().unwrap().build(&ids, &bounds);
+            *(self._bvh_invalidated.write().unwrap()) = false;
         }
     }
 
     fn build_winding_numbers(&self) {
         // Barill et al., Fast Winding Numbers for Soups and Clouds, ACM SIGGRAPH
         // 2018
-        if *self._wn_invalidated.borrow() {
+        if *self._wn_invalidated.read().unwrap() {
             self.build_bvh();
 
-            let n_nodes = self._bvh.borrow().number_of_nodes();
-            self._wn_area_weighted_normal_sums.borrow_mut().resize(n_nodes, Vector3D::new_default());
-            self._wn_area_weighted_avg_positions.borrow_mut().resize(n_nodes, Vector3D::new_default());
+            let n_nodes = self._bvh.read().unwrap().number_of_nodes();
+            self._wn_area_weighted_normal_sums.write().unwrap().resize(n_nodes, Vector3D::new_default());
+            self._wn_area_weighted_avg_positions.write().unwrap().resize(n_nodes, Vector3D::new_default());
 
             let mut visitor_func = |node_index: usize, data: &WindingNumberGatherData| {
-                self._wn_area_weighted_normal_sums.borrow_mut()[node_index] = data.area_weighted_normal_sums;
-                self._wn_area_weighted_avg_positions.borrow_mut()[node_index] = data.area_weighted_position_sums / data.area_sums;
+                self._wn_area_weighted_normal_sums.write().unwrap()[node_index] = data.area_weighted_normal_sums;
+                self._wn_area_weighted_avg_positions.write().unwrap()[node_index] = data.area_weighted_position_sums / data.area_sums;
             };
             let mut leaf_func = |node_index: usize| -> WindingNumberGatherData {
                 let mut result = WindingNumberGatherData::new();
 
-                let iter = self._bvh.borrow().item_of_node(node_index);
+                let iter = self._bvh.read().unwrap().item_of_node(node_index);
 
                 let tri = self.triangle(iter);
                 let area = tri.area();
@@ -601,10 +595,10 @@ impl TriangleMesh3 {
                 return result;
             };
 
-            post_order_traversal(&*self._bvh.borrow(), 0, &mut visitor_func, &mut leaf_func,
+            post_order_traversal(&*self._bvh.read().unwrap(), 0, &mut visitor_func, &mut leaf_func,
                                  &WindingNumberGatherData::new());
 
-            self._wn_invalidated.replace(false);
+            *(self._wn_invalidated.write().unwrap()) = false;
         }
     }
 
@@ -643,11 +637,11 @@ impl TriangleMesh3 {
                                     accuracy: f64) -> f64 {
         // Barill et al., Fast Winding Numbers for Soups and Clouds, ACM SIGGRAPH
         // 2018.
-        let tree_p = self._wn_area_weighted_avg_positions.borrow()[root_node_index];
+        let tree_p = self._wn_area_weighted_avg_positions.read().unwrap()[root_node_index];
         let q_to_p2 = q.distance_squared_to(tree_p);
 
-        let tree_n = self._wn_area_weighted_normal_sums.borrow()[root_node_index];
-        let tree_bound = self._bvh.borrow().node_bound(root_node_index);
+        let tree_n = self._wn_area_weighted_normal_sums.read().unwrap()[root_node_index];
+        let tree_bound = self._bvh.read().unwrap().node_bound(root_node_index);
         let tree_rvec =
             crate::vector3::max(&(tree_p - tree_bound.lower_corner), &(tree_bound.lower_corner - tree_p));
         let tree_r = tree_rvec.length();
@@ -659,13 +653,13 @@ impl TriangleMesh3 {
             // future.
             (tree_p - *q).dot(&tree_n) / (crate::constants::K_FOUR_PI_D * crate::math_utils::cubic(f64::sqrt(q_to_p2)))
         } else {
-            if self._bvh.borrow().is_leaf(root_node_index) {
+            if self._bvh.read().unwrap().is_leaf(root_node_index) {
                 // Case: q is nearby; use direct sum for tree’s elements
-                let iter = self._bvh.borrow().item_of_node(root_node_index);
+                let iter = self._bvh.read().unwrap().item_of_node(root_node_index);
                 self.winding_number(q, iter) * crate::constants::K_FOUR_PI_D
             } else {
                 // Case: Recursive call
-                let children = self._bvh.borrow().children(root_node_index);
+                let children = self._bvh.read().unwrap().children(root_node_index);
                 let mut wn = 0.0;
                 wn += self.fast_winding_number_internal(q, children.0, accuracy);
                 wn += self.fast_winding_number_internal(q, children.1, accuracy);
@@ -791,21 +785,21 @@ impl TriangleMesh3 {
                 let fv = 3;
                 for _ in 0..mesh.indices.len() / 3 {
                     if !mesh.indices.is_empty() {
-                        self.add_point_triangle(Size3::new(mesh.indices[next_face] as usize,
-                                                           mesh.indices[next_face + 1] as usize,
-                                                           mesh.indices[next_face + 2] as usize));
+                        self.add_point_triangle(USize3::new(mesh.indices[next_face] as usize,
+                                                            mesh.indices[next_face + 1] as usize,
+                                                            mesh.indices[next_face + 2] as usize));
                     }
 
                     if !mesh.normal_indices.is_empty() {
-                        self.add_normal_triangle(Size3::new(mesh.normal_indices[next_face] as usize,
-                                                            mesh.normal_indices[next_face + 1] as usize,
-                                                            mesh.normal_indices[next_face + 2] as usize));
+                        self.add_normal_triangle(USize3::new(mesh.normal_indices[next_face] as usize,
+                                                             mesh.normal_indices[next_face + 1] as usize,
+                                                             mesh.normal_indices[next_face + 2] as usize));
                     }
 
                     if !mesh.texcoord_indices.is_empty() {
-                        self.add_uv_triangle(Size3::new(mesh.texcoord_indices[next_face] as usize,
-                                                        mesh.texcoord_indices[next_face + 1] as usize,
-                                                        mesh.texcoord_indices[next_face + 2] as usize));
+                        self.add_uv_triangle(USize3::new(mesh.texcoord_indices[next_face] as usize,
+                                                         mesh.texcoord_indices[next_face + 1] as usize,
+                                                         mesh.texcoord_indices[next_face + 2] as usize));
                     }
                     next_face += fv;
                 }
@@ -860,7 +854,7 @@ impl Surface3 for TriangleMesh3 {
             return tri.closest_distance(pt);
         };
 
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {
                 panic!("No item found!");
@@ -890,7 +884,7 @@ impl Surface3 for TriangleMesh3 {
     fn bounding_box_local(&self) -> BoundingBox3D {
         self.build_bvh();
 
-        return self._bvh.borrow().bounding_box();
+        return self._bvh.read().unwrap().bounding_box();
     }
 
     /// ```
@@ -940,7 +934,7 @@ impl Surface3 for TriangleMesh3 {
             return result.distance;
         };
 
-        let query_result = self._bvh.borrow().closest_intersection(ray, &mut test_func);
+        let query_result = self._bvh.read().unwrap().closest_intersection(ray, &mut test_func);
         let mut result = SurfaceRayIntersection3::new();
         result.distance = query_result.distance;
         match query_result.item {
@@ -998,7 +992,7 @@ impl Surface3 for TriangleMesh3 {
             return tri.closest_distance(pt);
         };
 
-        let query_result = self._bvh.borrow().nearest(other_point, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point, &mut distance_func);
         match query_result.item {
             None => {
                 panic!("No item found!");
@@ -1049,7 +1043,7 @@ impl Surface3 for TriangleMesh3 {
             return tri.intersects(ray);
         };
 
-        return self._bvh.borrow().intersects_ray(ray_local, &mut test_func);
+        return self._bvh.read().unwrap().intersects_ray(ray_local, &mut test_func);
     }
 
     /// ```
@@ -1090,7 +1084,7 @@ impl Surface3 for TriangleMesh3 {
             return tri.closest_distance(pt);
         };
 
-        let query_result = self._bvh.borrow().nearest(other_point_local, &mut distance_func);
+        let query_result = self._bvh.read().unwrap().nearest(other_point_local, &mut distance_func);
         return query_result.distance;
     }
 
@@ -1125,25 +1119,13 @@ impl Surface3 for TriangleMesh3 {
         self.build_winding_numbers();
     }
 
-    fn transform(&self) -> &Transform3 {
-        return &self.transform;
-    }
-
-    fn set_transform(&mut self, t: Transform3) {
-        self.transform = t;
-    }
-
-    fn is_normal_flipped(&self) -> bool {
-        return self.is_normal_flipped;
-    }
-
-    fn set_is_normal_flipped(&mut self, f: bool) {
-        self.is_normal_flipped = f;
+    fn view(&self) -> &Surface3Data {
+        return &self.surface_data;
     }
 }
 
 /// Shared pointer for the TriangleMesh3 type.
-pub type TriangleMesh3Ptr = Rc<RefCell<TriangleMesh3>>;
+pub type TriangleMesh3Ptr = Arc<RwLock<TriangleMesh3>>;
 
 //--------------------------------------------------------------------------------------------------
 ///
@@ -1153,13 +1135,11 @@ pub struct Builder {
     _points: Vec<Vector3D>,
     _normals: Vec<Vector3D>,
     _uvs: Vec<Vector2D>,
-    _point_indices: Vec<Size3>,
-    _normal_indices: Vec<Size3>,
-    _uv_indices: Vec<Size3>,
+    _point_indices: Vec<USize3>,
+    _normal_indices: Vec<USize3>,
+    _uv_indices: Vec<USize3>,
 
-    _transform: Transform3,
-
-    _is_normal_flipped: bool,
+    _surface_data: Surface3Data,
 }
 
 impl Builder {
@@ -1182,19 +1162,19 @@ impl Builder {
     }
 
     /// Returns builder with point indices.
-    pub fn with_point_indices(&mut self, point_indices: Vec<Size3>) -> &mut Self {
+    pub fn with_point_indices(&mut self, point_indices: Vec<USize3>) -> &mut Self {
         self._point_indices = point_indices;
         return self;
     }
 
     /// Returns builder with normal indices.
-    pub fn with_normal_indices(&mut self, normal_indices: Vec<Size3>) -> &mut Self {
+    pub fn with_normal_indices(&mut self, normal_indices: Vec<USize3>) -> &mut Self {
         self._normal_indices = normal_indices;
         return self;
     }
 
     /// Returns builder with uv indices.
-    pub fn with_uv_indices(&mut self, uv_indices: Vec<Size3>) -> &mut Self {
+    pub fn with_uv_indices(&mut self, uv_indices: Vec<USize3>) -> &mut Self {
         self._uv_indices = uv_indices;
         return self;
     }
@@ -1204,7 +1184,7 @@ impl Builder {
     /// use vox_geometry_rust::vector3::Vector3D;
     /// use vox_geometry_rust::vector2::Vector2D;
     /// use vox_geometry_rust::triangle_mesh3::TriangleMesh3;
-    /// use vox_geometry_rust::size3::Size3;
+    /// use vox_geometry_rust::usize3::USize3;
     ///
     /// let points = vec![
     ///     Vector3D::new(1.0, 2.0, 3.0),
@@ -1226,18 +1206,18 @@ impl Builder {
     /// ];
     ///
     /// let point_indices = vec![
-    ///     Size3::new(0, 1, 2),
-    ///     Size3::new(0, 1, 3)
+    ///     USize3::new(0, 1, 2),
+    ///     USize3::new(0, 1, 3)
     /// ];
     ///
     /// let normal_indices = vec![
-    ///     Size3::new(1, 2, 3),
-    ///     Size3::new(2, 1, 0)
+    ///     USize3::new(1, 2, 3),
+    ///     USize3::new(2, 1, 0)
     /// ];
     ///
     /// let uv_indices = vec![
-    ///     Size3::new(1, 0, 2),
-    ///     Size3::new(3, 1, 0)
+    ///     USize3::new(1, 0, 2),
+    ///     USize3::new(3, 1, 0)
     /// ];
     ///
     /// let mesh = TriangleMesh3::builder()
@@ -1279,13 +1259,13 @@ impl Builder {
                                   self._point_indices.clone(),
                                   self._normal_indices.clone(),
                                   self._uv_indices.clone(),
-                                  Some(self._transform.clone()),
-                                  Some(self._is_normal_flipped));
+                                  Some(self._surface_data.transform.clone()),
+                                  Some(self._surface_data.is_normal_flipped));
     }
 
     /// Builds shared pointer of TriangleMesh3 instance.
     pub fn make_shared(&mut self) -> TriangleMesh3Ptr {
-        return TriangleMesh3Ptr::new(RefCell::new(self.build()));
+        return TriangleMesh3Ptr::new(RwLock::new(self.build()));
     }
 
     /// constructor
@@ -1296,25 +1276,14 @@ impl Builder {
             _uvs: vec![],
             _point_indices: vec![],
             _normal_indices: vec![],
-            _is_normal_flipped: false,
-            _transform: Transform3::new_default(),
+            _surface_data: Surface3Data::new(None, None),
             _uv_indices: vec![],
         };
     }
 }
 
 impl SurfaceBuilderBase3 for Builder {
-    fn with_is_normal_flipped(&mut self, is_normal_flipped: bool) -> &mut Self {
-        self._is_normal_flipped = is_normal_flipped;
-        return self;
-    }
-
-    fn with_transform(&mut self, transform: Transform3) -> &mut Self {
-        self._transform = transform;
-        return self;
-    }
-
-    fn transform(&mut self) -> &mut Transform3 {
-        return &mut self._transform;
+    fn view(&mut self) -> &mut Surface3Data {
+        return &mut self._surface_data;
     }
 }
\ No newline at end of file
